@@ -1,5 +1,5 @@
 from audiolm_pytorch.audiolm_pytorch import AudioLM
-from audiolm_pytorch.audiolm_pytorch import SoundStream
+from audiolm_pytorch.soundstream import SoundStream
 
 from audiolm_pytorch.audiolm_pytorch import SemanticTransformer, CoarseTransformer, FineTransformer
 from audiolm_pytorch.audiolm_pytorch import FineTransformerWrapper, CoarseTransformerWrapper
